/*******************************************************************************
  File: XIOStream.h
  Copyright (c) 1999 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee, 
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived from
  or uses this Software. You may not do anything however, to prevent the 
  continued distribution of this Software under an open source distribution
  model. This includes, but is not limited to, such things as copyrighting
  or claiming authorship of the Software.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
$Header: /osahome/Amber/RWE/libprjs/util/String_stream.h,v 1.2 2001/01/31 15:12:19 jreeves Exp $
  Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
  XDR_Stream is an iostream that reads and writes XDR.
  
//#----------------------------------------------------------------------------|
Notes:
  
//%****************************************************************************/
#ifndef UTIL_XIO_STREAM_H
#define UTIL_XIO_STREAM_H

#include <platform.h>

#include "XDRStream.h"

namespace Util {

class XIOStreambuf : public XDR_Streambuf {
	// typedefs
	typedef XDR_Streambuf					inherited;
	typedef inherited::traits_type			traits;
	typedef inherited::int_type				int_type;
	typedef traits::pos_type				pos_type;
	typedef traits::off_type				off_type;
	typedef std::streamsize					streamsize;

	// data
	std::streambuf*		_sb;
	char_type			_ibuf[128];		// so we can make underflow / uflow work properly
	char_type			_obuf[128]; 	// just to make things more efficient

public:
	XIOStreambuf(std::streambuf* sb);
	virtual ~XIOStreambuf();
	
	std::streambuf*			rdbuf()	const	{ return _sb; }

protected:
	virtual inherited*		setbuf(char* s, streamsize n);
	virtual pos_type		seekoff(off_type off, std::ios_base::seekdir dir,
									std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual pos_type		seekpos(pos_type pos,
									std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual int				sync();
	
	virtual int_type		underflow();
	
	virtual int_type		overflow(int_type c = traits::eof());
};

class iXIOStream : public iXDR_Stream {
	// data
	XIOStreambuf			_xsb;
	
public:
	iXIOStream(std::streambuf* sb);
	virtual ~iXIOStream();
	
private:
	iXIOStream(const iXIOStream&);
	iXIOStream& operator=(const iXIOStream&);
};
	


class oXIOStream : public oXDR_Stream {
	// data
	XIOStreambuf			_xsb;
	
public:
	oXIOStream(std::streambuf* sb);
	virtual ~oXIOStream();
	
private:
	oXIOStream(const oXIOStream&);
	oXIOStream& operator=(const oXIOStream&);
};
	

class XIOStream : public XDR_Stream {
	// data
	XIOStreambuf			_xsb;
	
public:
	XIOStream(std::streambuf* sb);
	virtual ~XIOStream();
	
private:
	XIOStream(const XIOStream&);
	XIOStream& operator=(const XIOStream&);
};

} //> namespace Util

#endif
