//
//  RandomMin.cpp - Implement RandomMin, RandomMax classes.
#include "stdafx.h"
#include "RandomMin.h"
#include "math.h"
#include "assert.h"

int randomNum(int maxValPlusOne)
{
   int r = rand();
   int rnum = (r*maxValPlusOne)/(RAND_MAX+1);
   return rnum;
}

//////////////////////////////////////////////////////////////////////
RandomMin::RandomMin()
{
   m_sampleSize= 3;
   m_tol=ROUNDOFF_ERR;
   m_vgList.clear();
   m_sampleCnt=0;
};

//////////////////////////////////////////////////////////////////////
void RandomMin::Initialize(int sampleSize)
{
   m_sampleSize= sampleSize;
   m_vgList.clear();
   m_sampleCnt=0;
}


int RandomMin::RandomSelectIndex()
{
   int rnum;
   if (m_sampleCnt < m_sampleSize)
      rnum = randomNum(m_sampleCnt);
   else
      rnum = randomNum(m_sampleSize);
   //
   // Find the index within the value group.
   // corresponding to this number.
   // This is the selected index we want to return.
   int base = 0;
   list <ValueGroup>::iterator vg = m_vgList.begin();
   while (vg != m_vgList.end())
   {
      int size = vg->m_index.size();
      int cnt = base + size;
      if (cnt > rnum)
      {
         if ((cnt < m_sampleCnt)||(m_sampleCnt <= m_sampleSize))
            return vg->m_index[rnum-base];
         else
         {
            //
            //  Need to do another random selection from
            //  the group of worst values because there
            //  were more than the sample size called for.
            rnum = randomNum(size);
            return vg->m_index[rnum];
         }
      }
      base = cnt;
      ++vg;
   }
   return -1;
}

void RandomMin::Compare( int index, double val)
{
   list <ValueGroup>::iterator vg = m_vgList.begin();
   if (vg == m_vgList.end())
   {
      //
      // Empty list. Add the new value.
      m_vgList.push_back( ValueGroup( index, val));
      m_sampleCnt++;
   }
   else if ((val - m_vgList.rbegin()->m_val) > m_tol)
   {
      //
      // New value is worse than current worst value.
      if (m_sampleCnt < m_sampleSize)
      {
         // But we need more samples.
         m_vgList.push_back( ValueGroup( index, val));
         m_sampleCnt++;
      }
   }
   else
   {
      if ((val - vg->m_val) < -m_tol)
      {
         //
         // The new value is better than the best value.
         m_vgList.push_front( ValueGroup(index, val));
         m_sampleCnt++;
      }
      else
      {
         //
         //  Value is somewhere in between best and worst.
         int oldSampleCnt = m_sampleCnt;
         while (vg != m_vgList.end())
         {
            if (fabs(val - vg->m_val) <= m_tol)
            {
               // essentially equal.
               vg->m_index.push_back(index); 
               m_sampleCnt++;
               break;
            }
            else if (val < vg->m_val)
            {
               m_vgList.insert( vg, ValueGroup(index, val));
               m_sampleCnt++;
               break;
            }
            ++vg;
         } 
         assert(m_sampleCnt-oldSampleCnt == 1);
      }
      list <ValueGroup>::reverse_iterator rvg = m_vgList.rbegin();
      int worstValGroupCnt = rvg->m_index.size();
      if ((m_sampleCnt - worstValGroupCnt) == m_sampleSize)
      {
         m_vgList.pop_back();
         m_sampleCnt -= worstValGroupCnt;
      }
      rvg = m_vgList.rbegin();
      worstValGroupCnt = rvg->m_index.size();
      assert((m_sampleCnt - worstValGroupCnt) < m_sampleSize);
   }
}

void RandomMax::Compare( int index, double val)
{
   list <ValueGroup>::iterator vg = m_vgList.begin();
   if (vg == m_vgList.end())
   {
      //
      // Empty list. Add the new value.
      m_vgList.push_back( ValueGroup( index, val));
      m_sampleCnt++;
   }
   else if ((val - m_vgList.rbegin()->m_val) < -m_tol)
   {
      //
      // New value is worse than current worst value.
      if (m_sampleCnt < m_sampleSize)
      {
         // But we need more samples.
         m_vgList.push_back( ValueGroup( index, val));
         m_sampleCnt++;
      }
   }
   else
   {
      if ((val - vg->m_val) > m_tol)
      {
         //
         // The new value is better than the best value.
         m_vgList.push_front( ValueGroup(index, val));
         m_sampleCnt++;
      }
      else
      {
         //
         //  Value is somewhere in between best and worst.
         int oldSampleCnt = m_sampleCnt;
         while (vg != m_vgList.end())
         {
            if (fabs(val - vg->m_val) <= m_tol)
            {
               // essentially equal.
               vg->m_index.push_back(index); 
               m_sampleCnt++;
               break;
            }
            else if (val > vg->m_val)
            {
               m_vgList.insert( vg, ValueGroup(index, val));
               m_sampleCnt++;
               break;
            }
            ++vg;
         } 
         assert(m_sampleCnt-oldSampleCnt == 1);
      }
      list <ValueGroup>::reverse_iterator rvg = m_vgList.rbegin();
      int worstValGroupCnt = rvg->m_index.size();
      if ((m_sampleCnt - worstValGroupCnt) == m_sampleSize)
      {
         m_vgList.pop_back();
         m_sampleCnt -= worstValGroupCnt;
      }
      rvg = m_vgList.rbegin();
      worstValGroupCnt = rvg->m_index.size();
      assert((m_sampleCnt - worstValGroupCnt) < m_sampleSize);
   }
}

