//
// TimeShareSolver.cpp : Implementation of CTimeShareSolver
#include "stdafx.h"
#include "Tssolve.h"
#include "TimeShareSolver.h"
// TimeShareSolver.h : CTimeShareSolver
//
#import "msado15.dll" rename("EOF", "adoEOF" ) no_namespace
#include "assert.h"
#include "RandomMin.h"
#include "math.h"
CTimeShareSolver::CTimeShareSolver()
{
   m_ownerCnt  = 0;
   m_owner     = NULL;
   m_trial     = NULL;
   m_trialCost = NULL;
   m_curr      = NULL;
   m_best      = NULL;
   m_trialCostTotal = 0;
   m_currCostTotal  = 0;
   m_bestCostTotal  = 0;
   memset( m_unitsOccup, 0, MAX_PERIODS*sizeof(int));
   memset( m_totalOccup, 0,  MAX_PERIODS*sizeof(int));
}

CTimeShareSolver::~CTimeShareSolver()
{
   if (m_ownerCnt != 0)
   {
      delete [] m_owner;
      delete [] m_trial;
      delete [] m_trialCost;
      delete [] m_curr;
      delete [] m_best;
   }
}

STDMETHODIMP CTimeShareSolver::Solve()
{
   //
   // Read the database.
   ReadOwnerInfo();
   //
   // Initialization.
   Initialize();
   //
   // Main loop.
   for (m_iterCnt =0;m_iterCnt<MAX_ITERATIONS;m_iterCnt++)
   {
      AssignOwners();
      SwapOwners();
      EvaluateCost();
      UnassignOwners();
   }
   //
   // Update the database.
   WriteBestSolution();
   return S_OK;
}

CTimeShareSolver::Initialize()
{
   //
   // Initialize all owners to unassigned.
   m_trialCostTotal = 0;
   for (int i = 0;i<m_ownerCnt;i++)
   {
      int cost = m_owner[i].CalcCost(UNASSIGNED);
      m_trial[i] = UNASSIGNED;
      m_curr[i] = UNASSIGNED;
      m_best[i] = UNASSIGNED;
      m_trialCost[i] = cost;
      m_trialCostTotal += cost;
   }
   //
   //  Initially the trial = curr = best soln.
   m_currCostTotal = m_trialCostTotal;
   m_bestCostTotal = m_trialCostTotal;
   //
   //  Initialize capacity data.
   for (int p =0;p < MAX_PERIODS;p++)
   {
      m_unitsOccup [p] = 0;
      m_totalOccup [p] = 0;
   }
   m_iterCnt = 0;
   srand(1); // Init random number generator.
}

CTimeShareSolver::AssignOwners()
{
   vector <bool> tryToAssign ( m_ownerCnt, false);
   int tryToAssignCnt = 0;
   for (int i=0;i<m_ownerCnt;i++)
   {
      if (m_trial[i] == UNASSIGNED)
      {
         tryToAssign[i] = true;
         tryToAssignCnt++;
      }
   }
   while (tryToAssignCnt > 0)
   {
      // Select one of the unassigned owners
      // with most occupants.
      RandomMax hiOcc;
      hiOcc.Initialize( 4);
      for (i=0;i<m_ownerCnt;i++)
      {
         if (tryToAssign[i])
            hiOcc.Compare(i, m_owner[i].m_occupCnt);
      }
      i = hiOcc.RandomSelectIndex();
      //
      //  Try to get one of the owner's first 3 choices.
      int choice = 0;
      while ((choice < 3)&&(m_trial[i] == UNASSIGNED))
      {
         int period = m_owner[i].m_choice[choice];
         if (m_unitsOccup[period] < MAX_UNITS)
         {
            int trialOcc = m_totalOccup[period];
            trialOcc +=  m_owner[i].m_occupCnt;
            if (trialOcc <= MAX_OCCUPANTS)
               Assign( i, period);
         }
         choice++;
      }
      //
      //  If owner is still unassigned. Try assigning him to
      //  the open period with the most occupants.
      if (m_trial[i] == UNASSIGNED)
      {
         int bestPeriod = -1;
         int maxOcc = -1;
         for (int period=0;period<MAX_PERIODS;period++)
         {
            if (m_unitsOccup[period] < MAX_UNITS)
            {
               int trialOcc = m_totalOccup[period];
               trialOcc += m_owner[i].m_occupCnt;
               if ((trialOcc <= MAX_OCCUPANTS)
                 &&(trialOcc > maxOcc))
               {
                  bestPeriod = period;
                  maxOcc = trialOcc;
               }
            }
         }
         if (bestPeriod != -1)
            Assign( i, bestPeriod);
      }
      tryToAssign[i] = false;
      tryToAssignCnt--;
   }
}

CTimeShareSolver::SwapOwners()
{
   vector <bool> tryToSwap ( m_ownerCnt, false);
   int tryToSwapCnt = 0;
   for (int i=0;i<m_ownerCnt;i++)
   {
      if (m_trialCost[i] > 0)
      {
         tryToSwap[i] = true;
         tryToSwapCnt++;
      }
   }
   while (tryToSwapCnt > 0)
   {
      RandomMax hiCost;
      hiCost.Initialize( 3);
      for (i=0;i<m_ownerCnt;i++)
      {
         // Select one of the unassigned owners
         // with highest cost.
         if (tryToSwap[i])
            hiCost.Compare(i, m_trialCost[i]);
      }
      i = hiCost.RandomSelectIndex();
      //
      //  We make swaps by unassigning first.
      //  Then reassign to someone else's spot.
      //  or back to the original spot.
      if (m_trial[i] != UNASSIGNED)
         Unassign( i);
      Swap(i);
      tryToSwap[i] = false;
      tryToSwapCnt--;
   }
}

CTimeShareSolver::UnassignOwners()
{
   int assignCnt  = 0;
   for (int i = 0;i<m_ownerCnt;i++)
   {
      if (m_trial[i] != UNASSIGNED)
         assignCnt++;
   }
   //
   // Unassign 15% of highest cost assignments.
   int unassignTarget = 15*assignCnt/100;
   while (unassignTarget > 0)
   {
      // Select one of the unassigned owners
      // with highest cost.
      RandomMax hiCost;
      hiCost.Initialize( 4);
      for (i=0;i<m_ownerCnt;i++)
      {
         if (m_trial[i] != UNASSIGNED)
            hiCost.Compare(i, m_trialCost[i]);
      }
      i = hiCost.RandomSelectIndex();
      if (m_trial[i] != UNASSIGNED)
      {
         Unassign(i);
         unassignTarget--;
      }
   }

   //
   // Remove an additional 5% at random.
   unassignTarget = 5*assignCnt/100;
   while (unassignTarget > 0)
   {
      int i=randomNum(m_ownerCnt);
      if (m_trial[i] != UNASSIGNED)
      {
         Unassign(i);
         unassignTarget--;
      }
   }
}


CTimeShareSolver::EvaluateCost()
{
   //
   //  First adjust the simulated annealing temperature;
   if (m_iterCnt== 0)
      m_saTemp = 0.05*m_trialCostTotal + 1.0;// initialize
   else if ((m_iterCnt % (MAX_ITERATIONS/20)) == 0)
      m_saTemp = 0.90*m_saTemp; // make 20 reductions of 10%
   //
   // First check for new best solution.
   if (m_trialCostTotal < m_bestCostTotal)
   {
      // A new best solution was found.
      // Copy trial solution to best solution.
      for (int i =0;i < m_ownerCnt;i++)
         m_best[i] = m_trial[i];
      m_bestCostTotal = m_trialCostTotal;
      //
      //  Copy trial solution to current solution.
      for (i =0;i < m_ownerCnt;i++)
         m_curr[i] = m_trial[i];
      m_currCostTotal = m_trialCostTotal;

      char buff[400];
      sprintf( buff, "Iteration %d NEW BEST SOLUTION = %d\n",
         m_iterCnt, m_bestCostTotal);
      OutputDebugString(buff);

   }
   else
   {
      double delta = m_currCostTotal - m_trialCostTotal;
      double anneal = exp(delta/m_saTemp);
      double randVal = randomNum(1000)/1000.0;
      if (randVal < anneal)
      {
         //
         //  Keep the trial solution.
         for (int i =0;i < m_ownerCnt;i++)
            m_curr[i] = m_trial[i];
         m_currCostTotal = m_trialCostTotal;
      }
      else
      {
         //
         // Discard trial soln. Go back to curr soln.
         for (int p =0;p < MAX_PERIODS;p++)
         {
            m_unitsOccup [p] = 0;
            m_totalOccup [p] = 0;
         }
         for (int i =0;i < m_ownerCnt;i++)
         {
            int p = m_curr[i];
            m_trial[i] = p;
            m_trialCost[i] = m_owner[i].CalcCost(p);
            if (p != UNASSIGNED)
            {
               m_unitsOccup[p]++;
               m_totalOccup[p] += m_owner[i].m_occupCnt;
            }
         }
         m_trialCostTotal = m_currCostTotal;
      }
   }
}

CTimeShareSolver::Swap(int owner1)
{
   int cost;
   RandomMin minSwapCost;
   minSwapCost.Initialize( 1); // no randomness here.
   for (int p1 = 0;p1 < MAX_PERIODS;p1++)
   {
      if ((m_unitsOccup[p1] < MAX_UNITS)
        &&(m_totalOccup[p1] + m_owner[owner1].m_occupCnt <= MAX_OCCUPANTS))
      {
         // Get cost of simple assignment. No swap.
         cost = m_owner[owner1].CalcCost(p1) - m_trialCost[owner1];
         minSwapCost.Compare( owner1*MAX_PERIODS + p1, cost);
      }
      else
      {
         //
         // Try to swap by reassigning someone else.
         for (int i=0;i<m_ownerCnt;i++)
         {
            if (m_trial[i] == p1)
            {
               int trialOcc = m_totalOccup[p1];
               trialOcc += m_owner[owner1].m_occupCnt;
               trialOcc -= m_owner[i].m_occupCnt;
               if (trialOcc <= MAX_OCCUPANTS)
               {
                  for (int p2=0;p2<MAX_PERIODS;p2++)
                  {
                     if ((p2 != p1)&&(m_unitsOccup[p2] < MAX_UNITS))
                     {
                        int trialOcc = m_totalOccup[p2];
                        trialOcc +=  m_owner[i].m_occupCnt;
                        if (trialOcc <= MAX_OCCUPANTS)
                        {
                           cost =  m_owner[owner1].CalcCost(p1) - m_trialCost[owner1];
                           cost += m_owner[i].CalcCost(p2) - m_trialCost[i];
                           minSwapCost.Compare( i*MAX_PERIODS + p2, cost);
                        }
                     }
                  }
               }
            }
         }
      }
   }
   int i = minSwapCost.RandomSelectIndex();
   if (i != -1)
   {
      int owner2  = i/MAX_PERIODS;
      int period2 = i % MAX_PERIODS;

      if (owner2 != owner1)
      {
         int period1 = m_trial[owner2];
         Unassign( owner2);
         Assign( owner2, period2);
         Assign( owner1, period1);
      }
      else
      {
         Assign( owner1, period2);
      }
   }
}


CTimeShareSolver::Assign(int ownerIdx, int period)
{
   int oldAssignment = m_trial[ownerIdx];
   assert(oldAssignment == UNASSIGNED);
   assert((period >= 0)&&(period < MAX_PERIODS));

   m_totalOccup[period] += m_owner[ownerIdx].m_occupCnt;
   assert(m_totalOccup[period] <= MAX_OCCUPANTS);

   m_unitsOccup[period]++;
   assert(m_unitsOccup[period] <= MAX_UNITS);

   int cost = m_owner[ownerIdx].CalcCost(period);
   m_trialCostTotal += cost - m_trialCost[ownerIdx];
   m_trialCost[ownerIdx] = cost;
   m_trial[ownerIdx] = period;
}

CTimeShareSolver::Unassign(int ownerIdx)
{
   int oldAssignment = m_trial[ownerIdx];
   assert(oldAssignment != UNASSIGNED);

   m_totalOccup[oldAssignment] -= m_owner[ownerIdx].m_occupCnt;
   assert( m_totalOccup[oldAssignment] >= 0);

   m_unitsOccup[oldAssignment]--;
   assert( m_unitsOccup[oldAssignment] >= 0);

   int cost = m_owner[ownerIdx].CalcCost(UNASSIGNED);
   m_trialCostTotal += cost - m_trialCost[ownerIdx];
   m_trialCost[ownerIdx] = cost;
   m_trial[ownerIdx] = UNASSIGNED;
}
//
// Read the database.
HRESULT CTimeShareSolver::ReadOwnerInfo()
{
   try
   {
      _RecordsetPtr pRs;
      pRs.CreateInstance(__uuidof(Recordset));
      //
      // Create the query.
      CComBSTR bstrQuery = L"select OwnerID, Occupancy, FirstChoice, SecondChoice, ThirdChoice from Owners order by Occupancy DESC, OwnerID";
      _variant_t vBuff= bstrQuery;
       //
       // Open the recordset;
      pRs->Open(vBuff, L"dsn=TSDATA;pwd=;",adOpenStatic, adLockOptimistic, adCmdUnknown);
       //
       // Get the row count;
      _variant_t cnt = pRs->GetRecordCount();
      m_ownerCnt = cnt.lVal;
      m_owner = new Owner [m_ownerCnt];
       //
       // Get the data.
      _variant_t vArr = pRs->GetRows( cnt);
      //
      //  Pull data out of array.
      long lFields, lRecords;
      VARIANT vRet;
      HRESULT hr = SafeArrayGetUBound(vArr.parray, 1, &lFields);
      hr = SafeArrayGetUBound(vArr.parray, 2, &lRecords);
      long ind[2];
      for(int i=0;i<=lRecords;i++)   //-- Next Row loop
      { 
         for(int j=0;j<=lFields;j++)   //-- Next Column loop
         {
            ind[0]=j; ind[1]=i;
            hr = SafeArrayGetElement(vArr.parray, ind, &vRet);
            //-- here you can use returned VARIANT vRet
            switch (j)
            {
               case 0:
               m_owner[i].m_ownerID         = vRet.lVal;
               break;

               case 1:
               m_owner[i].m_occupCnt   = vRet.lVal;
               break;

               case 2:
               m_owner[i].m_choice[0]     = vRet.lVal-1;
               break;

               case 3:
               m_owner[i].m_choice[1]     = vRet.lVal-1;
               break;

               case 4:
               m_owner[i].m_choice[2]     = vRet.lVal-1;
               break;
            }
         }
      }
      m_trial = new int [m_ownerCnt];
      m_curr = new int [m_ownerCnt];
      m_best = new int [m_ownerCnt];
      m_trialCost = new int [m_ownerCnt];
   }
   catch ( _com_error e )
   {
      return e.Error();
   }
   catch ( ... )
   {
      return E_FAIL;
   }
   return S_OK;
}


//
// Update the database.
CTimeShareSolver::WriteBestSolution()
{
   try
   {
      _RecordsetPtr pRs;
      pRs.CreateInstance(__uuidof(Recordset));
      //
      // Create the query.
      CComBSTR bstrQuery = L"select OwnerID, Period, Cost from Owners";
      _variant_t vBuff = bstrQuery;
       //
       // Open the recordset;
      pRs->Open(vBuff, L"dsn=TSDATA;pwd=;",adOpenStatic, adLockOptimistic, adCmdUnknown);
       //
       // Get the row count;
      _variant_t cnt = pRs->GetRecordCount();
      assert(cnt.lVal == m_ownerCnt);
      while (VARIANT_FALSE == pRs->adoEOF)
      {
         _variant_t itemNo = (long)0;
         int ownerID = pRs->Fields->GetItem(itemNo)->Value.lVal;
         int period=0;
         int cost=0;
         for (int i=0;i<m_ownerCnt;i++)
         {
            if (m_owner[i].m_ownerID == ownerID)
            {
               period = m_best[i];
               cost = m_owner[i].CalcCost(period);
               break;
            }
         }
         //
         //  Update the period
         itemNo = (long)1;
         _variant_t newVal = (long)(period + 1);
         pRs->Fields->GetItem(itemNo)->PutValue( newVal);
         //
         //  Update the cost.
         itemNo = (long)2;
         newVal = (long)cost;
         pRs->Fields->GetItem(itemNo)->PutValue( newVal);
         pRs->Update();
         pRs->MoveNext();   
      }
   }
   catch ( _com_error e )
   {
      return e.Error();
   }
   catch ( ... )
   {
      return E_FAIL;
   }
   return S_OK;
}

